@@ -23,6 +23,8 @@ class SugarJar
       @repo_config = SugarJar::RepoConfig.config
       SugarJar::Log.debug("Repoconfig: #{@repo_config}")
       @color = options['color']
+      @pr_autofill = options['pr_autofill']
+      @pr_autostack = options['pr_autostack']
       @feature_prefix = options['feature_prefix']
       @checks = {}
       @main_branch = nil
@@ -43,19 +45,31 @@ class SugarJar
       name = fprefix(name)
       die("#{name} already exists!") if all_local_branches.include?(name)
       base ||= most_main
-      base_pieces = base.split('/')
-      git('fetch', base_pieces[0]) if base_pieces.length > 1
+      # If our base is a local branch, don't try to parse it for a remote name
+      unless all_local_branches.include?(base)
+        base_pieces = base.split('/')
+        git('fetch', base_pieces[0]) if base_pieces.length > 1
+      end
       git('checkout', '-b', name, base)
+      git('branch', '-u', base)
       SugarJar::Log.info(
         "Created feature branch #{color(name, :green)} based on " +
         color(base, :green),
       )
     end
+    alias f feature
+
+    def subfeature(name)
+      assert_in_repo
+      SugarJar::Log.debug("Subfature: #{name}")
+      feature(name, current_branch)
+    end
+    alias sf subfeature
 
     def bclean(name = nil)
       assert_in_repo
       name ||= current_branch
-      name = fprefix(name) unless all_local_branches.include?(name)
+      name = fprefix(name)
       if clean_branch(name)
         SugarJar::Log.info("#{name}: #{color('reaped', :green)}")
       else
@@ -100,7 +114,7 @@ class SugarJar
       # and then add any featureprefix, and if _that_ is a branch
       # name, replace the last arguement with that
       name = args.last
-      bname = fprefix(name) unless all_local_branches.include?(name)
+      bname = fprefix(name)
       if all_local_branches.include?(bname)
         SugarJar::Log.debug("Featurepefixing #{name} -> #{bname}")
         args[-1] = bname
@@ -133,11 +147,14 @@ class SugarJar
 
     alias sl smartlog
 
-    def up
+    def up(branch = nil)
       assert_in_repo
+      branch ||= current_branch
+      branch = fprefix(branch)
       # get a copy of our current branch, if rebase fails, we won't
       # be able to determine it without backing out
       curr = current_branch
+      git('checkout', branch)
       result = gitup
       if result['so'].error?
         backout = ''
@@ -156,6 +173,8 @@ class SugarJar
         SugarJar::Log.info(
           "#{color(current_branch, :green)} rebased on #{result['base']}",
         )
+        # go back to where we were if we rebased a different branch
+        git('checkout', curr) if branch != curr
       end
     end
 
@@ -302,6 +321,7 @@ class SugarJar
     def smartpullrequest(*args)
       assert_in_repo
       assert_common_main_branch
+
       if dirty?
         SugarJar::Log.warn(
           'Your repo is dirty, so I am not going to create a pull request. ' +
@@ -309,9 +329,38 @@ class SugarJar
         )
         exit(1)
       end
+
       if gh?
+        curr = current_branch
+        base = tracked_branch
+        if @pr_autofill
+          num_commits = git(
+            'rev-list', '--count', curr, "^#{base}"
+          ).stdout.strip.to_i
+          if num_commits > 1
+            SugarJar::Log.debug(
+              "Not using --fill because there are #{num_commits} commits",
+            )
+          else
+            SugarJar::Log.info('Autofilling in PR from commit message')
+            args.unshift('--fill')
+          end
+        end
+        if subfeature?(base)
+          # nil is prompt, true is always, false is never
+          if @pr_autostack.nil?
+            $stdout.print(
+              'It looks like this is a subfeature, would you like to base ' +
+              "this PR on #{base}? [y/n] ",
+            )
+            ans = $stdin.gets.strip
+            args += ['--base', base] if %w{Y y}.include?(ans)
+          elsif @pr_autostack
+            args += ['--base', base]
+          end
+        end
         SugarJar::Log.trace("Running: gh pr create #{args.join(' ')}")
-        system(which('gh'), 'pr', 'create', '--fill', *args)
+        system(which('gh'), 'pr', 'create', *args)
       else
         SugarJar::Log.trace("Running: hub pull-request #{args.join(' ')}")
         system(which('hub'), 'pull-request', *args)
@@ -341,7 +390,7 @@ class SugarJar
 
       src = "origin/#{current_branch}"
       fetch('origin')
-      diff = git('diff', src).stdout
+      diff = git('diff', "..#{src}").stdout
       return unless diff && !diff.empty?
 
       puts "Will merge the following suggestions:\n\n#{diff}"
@@ -369,6 +418,9 @@ class SugarJar
     def fprefix(name)
       return name unless @feature_prefix
 
+      return name if name.start_with?(@feature_prefix)
+      return name if all_local_branches.include?(name)
+
       newname = "#{@feature_prefix}#{name}"
       SugarJar::Log.debug(
         "Munging feature name: #{name} -> #{newname} due to feature prefix",
@@ -712,11 +764,15 @@ class SugarJar
     end
 
     def all_local_branches
-      branches = []
-      git('branch', '--format', '%(refname)').stdout.lines.each do |line|
-        branches << branch_from_ref(line.strip)
+      git(
+        'branch', '--format', '%(refname)'
+      ).stdout.lines.map do |line|
+        branch_from_ref(line.strip)
       end
-      branches
+    end
+
+    def all_remotes
+      git('remote').stdout.lines.map(&:strip)
     end
 
     def safe_to_clean(branch)
@@ -794,14 +850,28 @@ class SugarJar
       SugarJar::Log.debug('Fetching upstream')
       fetch_upstream
       curr = current_branch
-      base = tracked_branch
+      # this isn't a hash, it's a named param, silly rubocop
+      # rubocop:disable Style/HashSyntax
+      base = tracked_branch(fallback: false)
+      # rubocop:enable Style/HashSyntax
+      unless base
+        SugarJar::Log.info(
+          'The brach we were tracking is gone, resetting tracking to ' +
+          most_main,
+        )
+        git('branch', '-u', most_main)
+        base = most_main
+      end
+      # If this is a subfeature based on a local branch which has since
+      # been deleted, 'tracked branch' will automatically return <most_main>
+      # so we don't need any special handling for that
       if !MAIN_BRANCHES.include?(curr) && base == "origin/#{curr}"
         SugarJar::Log.warn(
           "This branch is tracking origin/#{curr}, which is probably your " +
           'downstream (where you push _to_) as opposed to your upstream ' +
           '(where you pull _from_). This means that "sj up" is probably ' +
           'rebasing on the wrong thing and doing nothing. You probably want ' +
-          'to do a "git branch -u upstream".',
+          "to do a 'git branch -u #{most_main}'.",
         )
       end
       SugarJar::Log.debug('Rebasing')
@@ -812,6 +882,12 @@ class SugarJar
       }
     end
 
+    # determine if this branch is based on another local branch (i.e. is a
+    # subfeature). Used to figure out of we should stack the PR
+    def subfeature?(base)
+      all_local_branches.reject { |x| x == most_main }.include?(base)
+    end
+
     def rebase_in_progress?
       # for rebase without -i
       rebase_file = git('rev-parse', '--git-path', 'rebase-apply').stdout.strip
@@ -821,12 +897,13 @@ class SugarJar
       File.exist?(rebase_file) || File.exist?(rebase_merge_file)
     end
 
-    def tracked_branch
+    def tracked_branch(fallback: true)
       s = git_nofail(
         'rev-parse', '--abbrev-ref', '--symbolic-full-name', '@{u}'
       )
       if s.error?
-        most_main
+        fallback ? most_main : nil
+
       else
         s.stdout.strip
       end
@@ -844,9 +921,7 @@ class SugarJar
     def upstream
       return @remote if @remote
 
-      s = git('remote')
-
-      remotes = s.stdout.lines.map(&:strip)
+      remotes = all_remotes
       SugarJar::Log.debug("remotes is #{remotes}")
       if remotes.empty?
         @remote = nil
